@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.161 2008-09-24 12:22:16 yangtse Exp $
+ * $Id: setup.h,v 1.163 2009-02-12 20:48:44 danf Exp $
  ***************************************************************************/
 
 /*
@@ -355,7 +355,6 @@
 #    define sclose(x)         close_s(x)
 #    define select(n,r,w,x,t) select_s(n,r,w,x,t)
 #    define ioctl(x,y,z) ioctlsocket(x,y,(char *)(z))
-#    define IOCTL_3_ARGS
 #    include <tcp.h>
 #    ifdef word
 #      undef word
@@ -430,10 +429,6 @@
 #endif
 #endif
 
-#ifdef mpeix
-#define IOCTL_3_ARGS
-#endif
-
 #ifdef NETWARE
 int netware_init(void);
 #ifndef __NOVELL_LIBC__
@@ -460,7 +455,7 @@ int netware_init(void);
 #endif
 
 #if !defined(CURL_DISABLE_HTTP) && !defined(CURL_DISABLE_NTLM)
-#if defined(USE_SSLEAY) || defined(USE_WINDOWS_SSPI)
+#if defined(USE_SSLEAY) || defined(USE_WINDOWS_SSPI) || defined(USE_GNUTLS)
 #define USE_NTLM
 #endif
 #endif
