@@ -19,7 +19,7 @@
 # This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 # KIND, either express or implied.
 #
-# $Id: ftpserver.pl,v 1.97 2008-11-06 00:10:58 danf Exp $
+# $Id: ftpserver.pl,v 1.98 2008-12-08 20:20:51 bagder Exp $
 ###########################################################################
 
 # This is the FTP server designed for the curl test suite.
@@ -407,14 +407,12 @@ sub SIZE_command {
 
     my $size = $data[0];
 
-    if($size ne "") {
-        # we check for "" to be able to explictly set the size to 0 and yet have
-        # that send a 213
+    if($size) {
         if($size > -1) {
             sendcontrol "213 $size\r\n";
         }
         else {
-            sendcontrol "350 $testno: SIZE is not supported.\r\n";
+            sendcontrol "550 $testno: No such file or directory.\r\n";
         }
     }
     else {
@@ -427,7 +425,7 @@ sub SIZE_command {
             sendcontrol "213 $size\r\n";
         }
         else {
-            sendcontrol "350 $testno: SIZE is not supported.\r\n";
+            sendcontrol "550 $testno: No such file or directory.\r\n";
         }
     }
     return 0;
